package com.sun.media.sound;

import javax.sound.sampled.Mixer;
import javax.sound.sampled.spi.MixerProvider;

public final class DirectAudioDeviceProvider extends MixerProvider
{
    private static DirectAudioDeviceProvider.DirectAudioDeviceInfo[] infos;
    private static DirectAudioDevice[] devices;
    
    public DirectAudioDeviceProvider() {
        synchronized (DirectAudioDeviceProvider.class) {
            if (Platform.isDirectAudioEnabled()) {
                init();
            }
            else {
                DirectAudioDeviceProvider.infos = new DirectAudioDeviceProvider.DirectAudioDeviceInfo[0];
                DirectAudioDeviceProvider.devices = new DirectAudioDevice[0];
            }
        }
    }
    
    private static void init() {
        int nGetNumDevices = nGetNumDevices();
        if (DirectAudioDeviceProvider.infos == null || DirectAudioDeviceProvider.infos.length != nGetNumDevices) {
            DirectAudioDeviceProvider.infos = new DirectAudioDeviceProvider.DirectAudioDeviceInfo[nGetNumDevices];
            DirectAudioDeviceProvider.devices = new DirectAudioDevice[nGetNumDevices];
            for (int i = 0; i < DirectAudioDeviceProvider.infos.length; ++i) {
                DirectAudioDeviceProvider.infos[i] = nNewDirectAudioDeviceInfo(i);
            }
        }
    }
    
    @Override
    public Mixer.Info[] getMixerInfo() {
        synchronized (DirectAudioDeviceProvider.class) {
            Mixer.Info[] array = new Mixer.Info[DirectAudioDeviceProvider.infos.length];
            System.arraycopy(DirectAudioDeviceProvider.infos, 0, array, 0, DirectAudioDeviceProvider.infos.length);
            return array;
        }
    }
    
    @Override
    public Mixer getMixer(Mixer.Info obj) {
        synchronized (DirectAudioDeviceProvider.class) {
            if (obj == null) {
                int i = 0;
                while (i < DirectAudioDeviceProvider.infos.length) {
                    Mixer device = getDevice(DirectAudioDeviceProvider.infos[i]);
                    if (device.getSourceLineInfo().length > 0) {
                        return device;
                    }
                    ++i;
                }
            }
            int j = 0;
            while (j < DirectAudioDeviceProvider.infos.length) {
                if (DirectAudioDeviceProvider.infos[j].equals(obj)) {
                    return getDevice(DirectAudioDeviceProvider.infos[j]);
                }
                ++j;
            }
        }
        throw new IllegalArgumentException("Mixer " + obj.toString() + " not supported by this provider.");
    }
    
    private static Mixer getDevice(DirectAudioDeviceProvider.DirectAudioDeviceInfo directAudioDeviceInfo) {
        int index = directAudioDeviceInfo.getIndex();
        if (DirectAudioDeviceProvider.devices[index] == null) {
            DirectAudioDeviceProvider.devices[index] = new DirectAudioDevice(directAudioDeviceInfo);
        }
        return DirectAudioDeviceProvider.devices[index];
    }
    
    private static native int nGetNumDevices();
    
    private static native DirectAudioDeviceProvider.DirectAudioDeviceInfo nNewDirectAudioDeviceInfo(int p0);
    
    static {
        Platform.initialize();
    }
    
    static final class DirectAudioDeviceInfo extends Mixer.Info
    {
        private final int index;
        private final int maxSimulLines;
        private final int deviceID;
        
        private DirectAudioDeviceInfo(int index, int deviceID, int maxSimulLines, String name, String vendor, String str, String version) {
            super(name, vendor, "Direct Audio Device: " + str, version);
            this.index = index;
            this.maxSimulLines = maxSimulLines;
            this.deviceID = deviceID;
        }
        
        int getIndex() {
            return this.index;
        }
        
        int getMaxSimulLines() {
            return this.maxSimulLines;
        }
        
        int getDeviceID() {
            return this.deviceID;
        }
    }
}
